/*
   See "Principles of Parallel Programming", page 195 and also page 21.

   This is a very un-OpenMP style version of the count3's program.
   This program is very nearly a direct translation of the pthreads
   version into OpenMP. The result is too low level an OpenMP program
   that does not make good use of OpenMP's capabilities.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <omp.h>

int * array;
int length;
int count;

int count3s()
{
   count = 0;

   #pragma omp parallel shared(array, count, length)
   {
      int i;
      int count_p = 0;

      int p    = omp_get_num_threads();
      int rank = omp_get_thread_num();  // compare with MPI, SPMD style code
      int start = rank * (length / p);
      int end   = start + (length / p);

      // each thread handles its own part of the data
      for(i = start; i < end; i++)
      {
         if(array[i] == 3)
         {
            count_p++;
         }
      }

      // update the global counter with this thread's local counter
      #pragma omp critical
      {
         count += count_p;
      }
   }// end parallel construct

   return count;
}


int main(int argc, char *argv[])
{
   int i;
   length = 1048576;  /*  2^20  */

   array = calloc(length, sizeof(int));

   /* initialize the array with random integers between 0 and 9 */
   srand(time(NULL));  /* seed the random number generator with current time */
   for(i = 0; i < length; i++)
   {
      array[i] = rand()%10;
   }

   count3s();  /* do the actual calculation */

   printf("The number of 3's is %d\n", count);

   /* As a test, let us count 3's the slow, serial way. */
   count = 0;
   for(i = 0; i < length; i++)
      if(array[i] == 3)
         count++;
   printf("The number of 3's is %d\n", count);

   return 0;
}